#include <X11/Xlib.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#include "plug.h"
#include "socket.h"

void interrupt_handler(int);
void print_help(void);

int main(int argc, char** argv) {
    signal(SIGINT, interrupt_handler);

    if (argc > 1) {
        if (strcmp(argv[1], "send") == 0) {
            if (argc == 3) {
                socket_send(argv[2]);
                return 0;
            } else {
                print_help();
                return 1;
            }
        } else {
            print_help();
            printf("ERROR: Unknown subcommand\n");
            return 1;
        }
    }


    if (socket_init() != 0) {
        return 1;
    }

    setup_plugins("./modules");
    char* buffer;

    Display *display = XOpenDisplay(NULL);
    Window window = DefaultRootWindow(display);

    while (1) {
        sock_msg_t msg = socket_poll();

        switch (msg) {
            case SM_RELOAD: {
                reload_plugins();
            } break;
            case SM_NONE: break;
        }

        buffer = poll_plugins("|");
        XStoreName(display, window, buffer);
        XFlush(display);
        sleep(1);
    }


  XCloseDisplay(display);
  free_plugins();
  return 0;

}

void interrupt_handler(int sig) {
    (void) sig;
    printf("\nCaught SIGINT, exiting\n");
    free_plugins();
    socket_close();
    exit(0);
}

void print_help(void) {
    printf("Usage: dim [command]\n");
    printf("Commands:\n");
    printf("    send [MESSAGE] - Sends a message, available messages: [reload]\n");
}
